--[[notsoeasygametype - flexible configuration for gametypes
	based on easygametype v0.3.2 by goulart and fafabis
	edit by lonsfor
]]

local ezgt_servername = CV_RegisterVar({
	name = "ezgt_servername",
	defaultvalue = "",
	flags = CV_NETVAR
})

local function doEscapeCode(str)
	local escape = {
		{a = "\\128", b = "\128"},
		{a = "\\129", b = "\129"},
		{a = "\\130", b = "\130"},
		{a = "\\131", b = "\131"},
		{a = "\\132", b = "\132"},
		{a = "\\133", b = "\133"},
		{a = "\\134", b = "\134"},
		{a = "\\135", b = "\135"},
		{a = "\\136", b = "\136"},
		{a = "\\137", b = "\137"},
		{a = "\\138", b = "\138"},
		{a = "\\139", b = "\139"},
		{a = "\\140", b = "\140"},
		{a = "\\141", b = "\141"},
		{a = "\\142", b = "\142"},
		{a = "\\143", b = "\143"}
	}
	for _, v in pairs(escape)
		str = str:gsub(v.a, v.b)
	end
	return str
end

local ezgt_debug = CV_RegisterVar({
	name = "ezgt_debug",
	defaultvalue = "Off",
	PossibleValue = CV_OnOff
})

local ezgt_roundmax = CV_RegisterVar({
	name = "ezgt_roundmax",
	defaultvalue = "5",
	PossibleValue = {MIN = 1, MAX = 20},
	flags = CV_NETVAR
})

local gametypes = {}
local nextgameindex = nil
local currentgameindex = nil
local loaded = false
local roundsleft = 0
local roundstoplay = 2

local function isvotable(command) --check if command is votable
	if server.HMvotable and type(server.HMvotable) == "table" then 
		for _, votable in pairs(server.HMvotable) do
			if votable == command then
				return true
			end
		end
		return false
	end
end

local function addgametype(p, ...) --add gametypes, recomend to use wait command in between aditions to prevent the buffer overflow thing(like you should do woth hostmod :^))
	if p ~= server then
		CONS_Printf(p, "This is the Host's job")
		return
	end

	if ... == nil then
		CONS_Printf(p, "Add a gametype to the manager\nFormat example:\nname=Name,shortname=Shortname,on=var1 val|var2 val,off=var1 val|var2 val") --format example
		return
	end

	local newgame = table.concat({...}, " ")
	if ezgt_debug.value == 1 then
		CONS_Printf(p, newgame)
	end

	local gametypetable = {name="", shortname="", on={}, off={}}

	for k, v in string.gmatch (newgame, "([^,=]+)=([^,=]+)") do

		if k == "name" then
			gametypetable.name = v

		elseif k == "shortname" then
			gametypetable.shortname = v

		elseif k == "on" then
			for x, y in string.gmatch (v, "([^| ]+) ([^| ]+)") do
				if not CV_FindVar(x) then --make sure the command exists
					CONS_Printf(p, "The command ".. x .." was not found\nMake sure it's wad/pk3 is added and try again")
					return
				end

				table.insert(gametypetable.on, {variable=x, value=y})
			end

		elseif k == "off" then
			for x, y in string.gmatch (v, "([^| ]+) ([^|]+)") do
				if not CV_FindVar(x) then
					CONS_Printf(p, "The command ".. x .." was not found\nMake sure it's wad/pk3 is added and try again")
					return
				end

				table.insert(gametypetable.off, {variable=x, value=y})
			end
		end

	end
	
	if gametypetable.name == "" then
		CONS_Printf(p, "You didn't give the gametype its name")
		return
	end
	if #gametypetable.on == 0 then
		CONS_Printf(p, "You didn't give the gametype its On settings")
		return
	end
	if #gametypetable.off == 0 then
		CONS_Printf(p, "You didn't give the gametype its Off settings")
		return
	end

	table.insert(gametypes, gametypetable)

	local addmessege = "Ready: ".. gametypetable.name
	if gametypetable.shortname ~= "" then
		addmessege = $+ "(".. gametypetable.shortname ..")"
	end

	if ezgt_debug.value then
		addmessege = $+ "\nOn settings: "
		for _, b in pairs(gametypetable.on) do
			addmessege = $+ b.variable .." \""+ b.value+"\", "
		end
		addmessege = $+ "\nOff settings: "
		for _, b in pairs(gametypetable.off) do
			offsettings = $+ b.variable +" \"" +b.value+"\", "
		end
	end

	CONS_Printf(p, addmessege)
end

local function resetgametype(p, nukecode)
	if currentgameindex == nil and nextgameindex == nil and nukecode == nil then
		CONS_Printf(p, "No gametype currently running")

	elseif (p == server or IsPlayerAdmin(p)) then
		roundsleft = 0
		nextgameindex = nil
		loaded = false --if in intermition/votetime load again

		if nukecode ~= nil and nukecode == "full" then --the nuclear option. use in case of emergency.
			currentgameindex = "nuke"

			chatprint("\131*FULL RESET INITIETED")
			COM_BufInsertText(p, "wait 35; exitlevel")
		else
			chatprint("\131*Gametype will be reset next round", true)
		end

	else
		CONS_Printf(p,"You dont have permition to reset the gametype")
		if isvotable(p, "resetgametype") then
			CONS_Printf(p,"Try \130vote resetgametype \128instead")
		end
	end
end

local function setgametype(p, askedgametype, askedrounds)
	if askedgametype == nil and askedrounds == nil then
		local infostring = "Gametypes avelable: \n\n"
		if #gametypes ~= 0 then
			for _, namepair in pairs(gametypes) do
				infostring = $+ namepair.name
				if namepair.shortname ~= "" then
					infostring = $+ " (".. namepair.shortname ..")"
				end
				infostring = $+ "\n"
			end

			infostring = $+ "\nUse \130setgametype [gametype] \133[#rounds]\n                            *optional\n\130extendgametype \133[#rounds]\n\130resetgametype\128\n"

			if currentgameindex ~= nil then
				infostring = $+ "\nCurrent Gametype: ".. gametypes[currentgameindex].name .." ("..roundsleft.." rounds)"
			end
			if nextgameindex ~= nil then
				infostring = $+ "\nQueued Gametype: ".. gametypes[nextgameindex].name .." ("..roundstoplay.." rounds)"
			end

		else
			infostring = $+ "none :("
		end

		CONS_Printf(p, infostring)

	elseif (p == server or IsPlayerAdmin(p)) then
		if askedgametype == "" then return end --this should never happen but you never know

		if nextgameindex ~= nil then
			if isvotable("setgametype") then
				chatprint("\131*A gametype is already queued.", true)
			else
				CONS_Printf(p, "A gametype is already queued.")
			end

		elseif currentgameindex ~= nil and roundsleft >= 1 then
			if isvotable("setgametype") then
				chatprint("\131*A gametype is already running, wait for it to finish.", true)
			else
				CONS_Printf(p, "A gametype is already running, wait for it to finish.")
			end

		else
			if ezgt_debug.value == 1 then
				CONS_Printf(p, "DEBUG: searching for ".. askedgametype)
				if askedrounds ~= nil then
					CONS_Printf(p, askedrounds.." rounds")
				end
			end

			askedgametype = askedgametype:lower()

			for gametypeindex, namepair in pairs(gametypes) do --look for the names/shortnames of the gametypes
				if askedgametype == namepair.name:lower() or askedgametype == namepair.shortname:lower() then
					if type(gametypeindex) ~= "number" then
						CONS_Printf(server, "OH NO some fuckup shit happened")
					else
						nextgameindex = gametypeindex

						if ezgt_debug.value == 1 then
							CONS_Printf(server, "DEBUG: nextgameindex set to ".. nextgameindex)
						end

						askedrounds = tonumber(askedrounds)
						if askedrounds ~= nil and r > 0 then
							roundstoplay = min(ezgt_roundmax.value, askedrounds)
						else
							roundstoplay = 2
						end

						loaded = false --in case setgametype happens during intermission/vote

						local chatstring = "\131*Next gametype will be \130".. namepair.name .. "\131 for \130".. roundstoplay
						if roundstoplay == 1 then
							chatstring = $+ " round"
						else
							chatstring = $+ " rounds"
						end
						chatprint(chatstring, true)

						return
					end

				end

			end
			if isvotable("setgametype") then --no way to check if a command was done with vote so maybe this will do
				chatprint("\131*No gametype by the name. Check your spelling?", true)
			else
				CONS_Printf(p, "No gametype by that name. Check your spelling?")
			end
		end

	else
		CONS_Printf(p,"You dont have permition to set a gametype")
		if isvotable("setgametype") then
			CONS_Printf(p,"Try \130vote setgametype \128instead")
		end
	end
end

local function extendgametype(p, roundnum)
	roundnum = tonumber(roundnum)
	if roundnum == nil or roundnum <= 0 then
		CONS_Printf(p, "Extend the ammount of rounds to play the current gametype")
	else
		if (p == server or IsPlayerAdmin(p)) then
			if currentgameindex == nil then
				CONS_Printf(p, "No gametype currently running")
			else
				local r = roundsleft + roundnum
				roundsleft = min(ezgt_roundmax.value, r)

				local chatstring = "\131*Gametype extended to \130".. roundsleft
				if roundsleft == 1 then
					chatstring = $+ " round"
				else
					chatstring = $+ " rounds"
				end
				chatprint(chatstring, true)
			end
		else
			CONS_Printf(p,"You dont have permition to extend the gametype")
			if isvotable("extendgametype") then
				CONS_Printf(p,"Try \130vote extendgametype \128instead")
			end
		end
	end
end

local function newmap()
	loaded = false
	nextgameindex = nil

	if currentgameindex ~= nil then
		if roundsleft == 1 then
			COM_BufInsertText(server, "wait 35; csay ".. gametypes[currentgameindex].name .."\\Last Round")
		else
			COM_BufInsertText(server, "wait 35; csay ".. gametypes[currentgameindex].name .."\\".. tostring(roundsleft) .." Rounds Left")
		end
	end

	roundsleft = max($-1, 0)
end

local function loadnexgame()
	if loaded == true then return end

	if roundsleft == 0 and currentgameindex ~= nil then --unload gametype
		if currentgameindex == "nuke" then --nuclear option
			if ezgt_debug.value == 1 then
				CONS_Printf(server, "DEBUG: NUCEAR OPTION ENGAGED")
			end
			for _, gametypetable in pairs(gametypes) do
				for _, varvalpair in pairs(gametypetable.off) do --run through every gametype and set it off
					COM_BufInsertText(server, varvalpair.variable .." \"".. varvalpair.value .."\"")
					if ezgt_debug.value == 1 then
						CONS_Printf(server, varvalpair.variable .." set to \"".. varvalpair.value .."\"")
					end
				end
			end
			if ezgt_debug.value == 1 then
				CONS_Printf(server, "")
			end
		else --normal unload
			if ezgt_debug.value == 1 then
				CONS_Printf(server, "DEBUG: unloading ".. gametypes[currentgameindex].name)
			end
			for _, varvalpair in pairs(gametypes[currentgameindex].off) do
				COM_BufInsertText(server, varvalpair.variable .." \"".. varvalpair.value .."\"")
				if ezgt_debug.value == 1 then
					CONS_Printf(server, varvalpair.variable .." set to \"".. varvalpair.value .."\"")
				end
			end
			if ezgt_debug.value == 1 then
				CONS_Printf(server, "")
			end
		end
		currentgameindex = nil

		if ezgt_servername.string ~= "" then --turn back server name
			COM_BufInsertText(server, 'servername "'.. doEscapeCode(ezgt_servername.string) ..'"')
			if ezgt_debug.value == 1 then
				CONS_Printf(server, 'server name set to: "'.. doEscapeCode(ezgt_servername.string) ..'"')
			end
		end

		if ezgt_debug.value == 1 then
			CONS_Printf(server, "DEBUG: unload done")
		end
	end

	if nextgameindex ~= nil then --load new gametype
		if ezgt_debug.value == 1 then
			CONS_Printf(server, "DEBUG: loading ".. gametypes[nextgameindex].name)
		end
		for _, varvalpair in pairs(gametypes[nextgameindex].on) do
			COM_BufInsertText(server, varvalpair.variable .." \"".. varvalpair.value .."\"")
			if ezgt_debug.value == 1 then
				CONS_Printf(server, varvalpair.variable .." set to \"".. varvalpair.value .."\"")
			end
		end
		currentgameindex = nextgameindex
		roundsleft = roundstoplay

		if ezgt_servername.string ~= "" then --change server name
			if gametypes[nextgameindex].shortname ~= "" then --if gametype doesnt have a shortname use the normal name
				COM_BufInsertText(server, 'servername "'.. doEscapeCode(ezgt_servername.string) ..' \128| \132'.. gametypes[nextgameindex].shortname ..'"')
				if ezgt_debug.value == 1 then
					CONS_Printf(server, 'server name set to: "'.. doEscapeCode(ezgt_servername.string) ..' \128| \132'.. gametypes[nextgameindex].shortname ..'"')
				end
			else
				COM_BufInsertText(server, 'servername "'.. doEscapeCode(ezgt_servername.string) ..' \128| \132'.. gametypes[nextgameindex].name ..'"')
				if ezgt_debug.value == 1 then
					CONS_Printf(server, 'server name set to: "'.. doEscapeCode(ezgt_servername.string) ..' \128| \132'.. gametypes[nextgameindex].name ..'"')
				end
			end
		end

		if ezgt_debug.value == 1 then
			CONS_Printf(server, "DEBUG: load done")
		end
	end

	loaded = true
	if ezgt_debug.value == 1 then
		CONS_Printf(server, "DEBUG: next game ready")
	end
end

addHook("PlayerQuit", function(quiter) --reset gametype when no players are in the server
	if currentgameindex == nil and nextgameindex == nil then return end --if there is no gametype running/queued then do nothing

	for p in players.iterate do
		if p == quiter then
			continue
		elseif p == server and (p.mo == nil or p.spectator) then --is host dedicated/spec
			continue
		else
			return --a player was found, do nothing.
		end
	end
	CONS_Printf(server, "No players. resetting gametype")
	COM_BufInsertText(server, "resetGametype")
	COM_BufInsertText(server, "exitlevel")
end)

COM_AddCommand("addgametype", addgametype, 1)
COM_AddCommand("setgametype", setgametype)
COM_AddCommand("resetgametype", resetgametype)
COM_AddCommand("extendgametype", extendgametype)
COM_AddCommand("listgametypes", function(p)
	setgametype(p)
end)

addHook("MapLoad", newmap)
addHook("IntermissionThinker", loadnexgame)
addHook("VoteThinker", loadnexgame)

addHook("NetVars", function(n)
	gametypes = n($)
	currentgameindex = n($)
	nextgameindex = n($)
	loaded = n($)
	roundsleft = n($)
	roundstoplay = n($)
end)
